/////////////////////////////////////////////////////////////////////////////
//                                                                           
// Includes


#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoTransform.h>

#include "grid.h"


/////////////////////////////////////////////////////////////////////////////
//                                                                            
// Global Variables

extern  GridStruct       Grid;



/////////////////////////////////////////////////////////////////////////////
//                                                                            
//  WWWWW                                                                     
// WW  WW                         WW                                          
// WW                             WW                                          
// WW      WW WW   WWW    WWWW   WWWW   WWW                                   
// WW      WWW    WW WW      WW   WW   WW WW                                  
// WW      WW     WWWWW   WWWWW   WW   WWWWW                                  
// WW  WW  WW     WW     WW  WW   WW   WW                                     
//  WWWW   WW      WWWW   WWWWW   WW    WWWW                                  
//                                                                            
//                                                                            
//  WWWW                                                                      
// WW  WW                                                                     
// WWW                                                                        
//  WWWW    WWW    WWW   WWWW    WWW                                          
//    WWW  WW WW  WW WW  WW WW  WW WW                                         
//     WW  WW     WWWWW  WW WW  WWWWW                                         
// WW  WW  WW WW  WW     WW WW  WW                                            
//  WWWW    WWW    WWWW  WW WW   WWWW                                         
//                                                                            
//                                                                            
// Create a scene graph with draggers that control waves
// in a 2D mesh.

SoSeparator *
createScene()
  {
  //
  //                              root
  //                                |
  //    -----------------------------
  //    |    |   |        |     |
  // view  com  two      mesh  quad
  //xform  plex side     co    mesh
  //       ity           ord  
  //                   

  SoSeparator *root = new SoSeparator();

  // view orientation
  // Rotate the scene so that the z-axis is nearly vertical,
  // toward the y-direction. Rotate about the x-axis.
  SoTransform *viewTransform = new SoTransform();
    viewTransform->rotation.setValue(SbRotation
      (SbVec3f (1.0, 0.0, 0.0),  -M_PI*0.30) );
    viewTransform->rotation.setValue(
      viewTransform->rotation.getValue()  *
      SbRotation(SbVec3f (0.0, 1.0, 0.0),   M_PI*0.10 ) 
      ); 
  root->addChild(viewTransform);


  // complexity
  SoComplexity *complexity = new SoComplexity();
    complexity->textureQuality = 1.0;
    complexity->value = 0.8;
  root->addChild(complexity);

  // INVENTOR HACK: set up the hints to produce two-sided lighting
  SoShapeHints *twoSided = new SoShapeHints();
    twoSided->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
    twoSided->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
  root->addChild(twoSided);


  // mesh coordinates
  SoCoordinate3 *coord = new SoCoordinate3();
    coord->point.setNum(Grid.sDim * Grid.tDim);
  root->addChild(coord);
  Grid.coord = coord;


  // quadMesh
  SoQuadMesh *quadMesh = new SoQuadMesh;
    quadMesh->verticesPerRow    = Grid.sDim;
    quadMesh->verticesPerColumn = Grid.tDim;
  root->addChild(quadMesh);
  Grid.mesh = quadMesh;


  return root;
  }

